"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var vtk_1 = require("./vtk");
exports.VTKPlot = vtk_1.VTKPlot;
var vtkvolume_1 = require("./vtkvolume");
exports.VTKVolumePlot = vtkvolume_1.VTKVolumePlot;
var vtkaxes_1 = require("./vtkaxes");
exports.VTKAxes = vtkaxes_1.VTKAxes;
//# sourceMappingURL=index.js.map